#
# INTEL CONFIDENTIAL
#
# Copyright 2013-2017 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all documents related
# to the source code ("Material") are owned by Intel Corporation or its
# suppliers or licensors. Title to the Material remains with Intel Corporation
# or its suppliers and licensors. The Material contains trade secrets and
# proprietary and confidential information of Intel or its suppliers and
# licensors. The Material is protected by worldwide copyright and trade secret
# laws and treaty provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted, distributed,
# or disclosed in any way without Intel's prior express written permission.
#
# No license under any patent, copyright, trade secret or other intellectual
# property right is granted to or conferred upon you by disclosure or delivery
# of the Materials, either expressly, by implication, inducement, estoppel or
# otherwise. Any license under such intellectual property rights must be
# express and approved by Intel in writing.

import os
import re
import settings

from chroma_core.lib.util import CommandLine
cl = CommandLine()
try_shell = cl.try_shell

from functools import partial

crypto_folder = partial(os.path.join, settings.CRYPTO_FOLDER)
AUTHORITY_KEY_FILE = crypto_folder('authority.pem')
AUTHORITY_CERT_FILE = crypto_folder('authority.crt')
MANAGER_KEY_FILE = crypto_folder('manager.pem')
MANAGER_CERT_FILE = crypto_folder('manager.crt')
CERTIFICATE_DAYS = "36500"

def create_private_key(filename):
    try_shell(['openssl', 'genrsa', '-out', filename, '2048', '-sha256'])
    print '    - Generated Private Key: %s' % filename

def create_authority_key_and_cert():
    def create_authority_key():
        create_private_key(AUTHORITY_KEY_FILE)

    def create_authority_cert():
        rc, csr, err = try_shell(["openssl", "req", "-new", "-sha256", "-subj", "/C=/ST=/L=/O=/CN=x_local_authority", "-key", AUTHORITY_KEY_FILE])
        rc, out, err = try_shell(["openssl", "x509", "-req", "-sha256", "-days", CERTIFICATE_DAYS, "-signkey", AUTHORITY_KEY_FILE, "-out", AUTHORITY_CERT_FILE], stdin_text = csr)
        print '    - Created Authority Certificate using sha256: %s' % AUTHORITY_CERT_FILE

    create_authority_key()
    create_authority_cert()


def create_manager_key_and_cert(name):
    def create_manager_key():
        return create_private_key(MANAGER_KEY_FILE)

    def create_manager_cert(name):
        rc, csr, err = try_shell(["openssl", "req", "-new", "-sha256", "-subj", "/C=/ST=/L=/O=/CN=%s" % name, "-key", MANAGER_KEY_FILE])
        rc, out, err = try_shell(["openssl", "x509", "-req", "-sha256", "-days", CERTIFICATE_DAYS, "-CA", AUTHORITY_CERT_FILE, "-CAcreateserial", "-CAkey", AUTHORITY_KEY_FILE, "-out", MANAGER_CERT_FILE], stdin_text = csr)
        print "    - Created Manager Certificate using sha256: %s" % MANAGER_CERT_FILE

    create_manager_key()
    create_manager_cert(name)
